/* for phago TYPE 1; mito TYPE 2; er TYPE 3; other(notleish) TYPE 0; all no leishmania TYPE 4 (line 16);leish only TYPE = 5 */ 
/* use NPEP as the minimum number of peptides to be considered (line 17)*/
/* for scaled in pairs use PAIRS 1 otherwise, for scaled across 3 experiments, use PAIRS 0 (line 18)*/
/* change output file names on lines 46-48 */
/* this version normalises according to the proportion of each type of protein (0,1,2, or 3 */

#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#define MAXIT 100
#define EPS 3.0e-7
#define FPMIN 1.0e-30
#define TYPE 5
#define NPEP 1
#define PAIRS 1

typedef struct alldata{
  char name[1024];
  int protnum;
  int prottype;
  int speciesnum;
  double *vars;
  double meanpval;
  double pval1to3;
  double pval1;
  double pval2;
  double pval3;
  double pval4;  
}DATA;

float pvalue( float t, float df );

/****************************************
Procedure:      main
 *****************************************/
int main (int argc, char *argv[])
{  
    FILE *fin = NULL;
    FILE *fout1 = NULL;
    FILE *fout2 = NULL;
    FILE *fout3 = NULL;

    fout1 = fopen("c224pepdata_othernoleish_pairs_atleast3_newnorm.txt", "w");
    fout2 = fopen("c224protdata_othernoleish_pairs_atleast3_newnorm.txt", "w");
    fout3 = fopen("c224ttests_othernoleish_pairs_atleast3_newnorm.txt", "w");

    int i, j, k, n, tmp;
    int dtmp, dtmp1, dtmp2;
    float ftmp;
    int nvar = 8;
    int nobs = 6164;
    int nprot = 1088;
    char name[1024];
 
    double min, sum;
    double *tot = NULL; 
    tot = (double *) malloc (nvar * sizeof(double));
    
    double percentphago[nvar];   
    double percentmito[nvar];   
    double percentother[nvar];   
    double percenter[nvar];   
    double percentleish[nvar];   
       
    DATA *pep = NULL;
    pep = (DATA *) malloc (nobs * sizeof(DATA));
    for (i = 0; i < nobs; i++)
    {
      pep[i].vars = (double*) malloc (nvar * sizeof(double));     
    }

    DATA *prot = NULL;
    prot = (DATA *) malloc (nprot * sizeof(DATA));
    for (i = 0; i < nprot; i++)
    {
      prot[i].vars = (double*) malloc (nvar * sizeof(double));     
      for (k = 0; k < nvar; k++)
      {
         prot[i].vars[k] = 0.0;
      }
      prot[i].protnum = 0;
    }

    fin = fopen("c224las5.txt", "r");

    for (k = 0; k < nvar; k++)
    {
      percentphago[k] = 0.0;
      percentmito[k] = 0.0;
      percenter[k] = 0.0;
      percentother[k] = 0.0;
      percentleish[k] = 0.0;
    }

    for (i = 0; i < nobs; i++)
    {
      /* read in data */
      tmp = fscanf(fin, "%s %d %d %d ", name, &dtmp, &dtmp1, &dtmp2);
      pep[i].prottype = dtmp;      
      pep[i].protnum = dtmp1-1;
      pep[i].speciesnum = dtmp2;
      strcpy(pep[i].name, name);
      /*find minimum level and subtract */
      min = 0.0;
      for (k = 0; k < nvar; k++)
      {
        tmp = fscanf(fin, "%f ", &ftmp);
        pep[i].vars[k] = ftmp;
        if (pep[i].vars[k] < min) min = pep[i].vars[k]; 
      } 
      for (k = 0; k < nvar; k++)
      {
         pep[i].vars[k] += -min;
      }
    }
    fclose(fin);

    /* normalise */      
    for (k = 0; k < nvar; k++)
    {
      tot[k] = 0.0;
      for (i = 0; i < nobs; i++)
      {
         tot[k] += pep[i].vars[k]; 
         if (pep[i].prottype == 1) percentphago[k] += pep[i].vars[k];
         if (pep[i].prottype == 2) percentmito[k] += pep[i].vars[k];
         if (pep[i].prottype == 3) percenter[k] += pep[i].vars[k];
         if (pep[i].prottype == 0) percentother[k] += pep[i].vars[k];
         if (pep[i].prottype == 5) percentleish[k] += pep[i].vars[k];
      }
      for (i = 0; i < nobs; i++)
      {      
          if (pep[i].prottype == 1) pep[i].vars[k] *= 100.0*percentphago[k]/tot[k];
          if (pep[i].prottype == 2) pep[i].vars[k] *= 100.0*percentmito[k]/tot[k];
          if (pep[i].prottype == 3) pep[i].vars[k] *= 100.0*percenter[k]/tot[k];
          if (pep[i].prottype == 0) pep[i].vars[k] *= 100.0*percentother[k]/tot[k];
          if (pep[i].prottype == 5) pep[i].vars[k] *= 100.0*percentother[k]/tot[k];
      }      
    }

    if (PAIRS == 0)
    {
      /* calculate relative intensity for each peptide */
      for (i = 0; i < nobs; i++)
      {
        sum = 0.0;
        for (k = 0; k < nvar-2; k++)
        {
          sum += pep[i].vars[k];
        }
        for (k = 0; k < nvar; k++)
        {
           pep[i].vars[k] *= 100.0/sum;
        }             
      } 
    }
    else
    {
     /* calculate relative intensity for each pair in each peptide */
      for (i = 0; i < nobs; i++)
      {
        for (k = 0; k < nvar/2; k++)
        {        
          sum = pep[i].vars[2*k] + pep[i].vars[2*k+1];
          pep[i].vars[2*k] *= 1.0/sum;
          pep[i].vars[2*k+1] *= 1.0/sum;
        }
      }    
    }
    
    /* calculate mean for each protein */
    for (i = 0; i < nobs; i++)
    {
      for (k = 0; k < nvar; k++)
      {
         prot[pep[i].protnum].vars[k] += pep[i].vars[k];
      }
      prot[pep[i].protnum].protnum++;
      strcpy(prot[pep[i].protnum].name, pep[i].name);
      prot[pep[i].protnum].speciesnum = pep[i].speciesnum;
      prot[pep[i].protnum].prottype = pep[i].prottype;
    }
    for (i = 0; i < nprot; i++)
    {
      for (k = 0; k < nvar; k++)
      {
        prot[i].vars[k] /= (float)prot[i].protnum;
      }
    }
     
    float xmean, sigma, tval, pval;
    /* t-tests on protein means */
    for (i = 0; i < nprot; i++)
    {
      xmean = (prot[i].vars[0]-prot[i].vars[1] + prot[i].vars[2]-prot[i].vars[3] + prot[i].vars[4]-prot[i].vars[5])/3.0;
      sigma =  ((prot[i].vars[0]-prot[i].vars[1])*(prot[i].vars[0]-prot[i].vars[1]) + (prot[i].vars[2]-prot[i].vars[3])*(prot[i].vars[2]-prot[i].vars[3]) + (prot[i].vars[4]-prot[i].vars[5])*(prot[i].vars[4]-prot[i].vars[5]))/2.0;
      sigma += -1.5*xmean*xmean;
      tval = xmean/(sqrt(sigma/3.0));
      pval = pvalue(tval, 2.0);
      /* if (pval < 0.05) printf("%d %s %0.5f\n", i, prot[i].name, pval); */
      prot[i].meanpval = pval;
    }
        
    /* get combined pvalues */
    int protnum = 0;
    while (protnum < nprot)
    {
      xmean = 0.0;
      sigma = 0.0;
      n = 0;
      for (i = 0; i < nobs; i++)
      {
        if (pep[i].protnum == protnum)
        {
           if ((pep[i].vars[0] > 0.0) &&(pep[i].vars[1] > 0.0))
           {
             xmean += pep[i].vars[0] - pep[i].vars[1];
             sigma += (pep[i].vars[0] - pep[i].vars[1])*(pep[i].vars[0] - pep[i].vars[1]);
             n++;
           }
           if ((pep[i].vars[2] > 0.0) &&(pep[i].vars[3] > 0.0))
           {
             xmean += pep[i].vars[2] - pep[i].vars[3];
             sigma += (pep[i].vars[2] - pep[i].vars[3])*(pep[i].vars[2] - pep[i].vars[3]);
             n++;
           } 
           if ((pep[i].vars[4] > 0.0) &&(pep[i].vars[5] > 0.0))
           {
             xmean += pep[i].vars[4] - pep[i].vars[5];
             sigma += (pep[i].vars[4] - pep[i].vars[5])*(pep[i].vars[4] - pep[i].vars[5]);
             n++;
           }
        }
      }
      if (n > 1)
      {
        xmean /= (float)n;
        sigma /= (float)(n-1);
        sigma += -(float)n*xmean*xmean/(float)(n-1);
        tval = xmean/(sqrt(sigma/(float)n));
        pval = pvalue(tval, (float)(n-1));
        /* if (pval < 0.05) printf("%d %s %d %0.5f\n", protnum, prot[protnum].name, n, pval); */
        prot[protnum].pval1to3 = pval;
      }
      protnum++;
    } 

    /* get p values for individual experiments */
    for (j = 0; j < 4; j++)
    {
      protnum = 0;
      while (protnum < nprot)
      {
        xmean = 0.0;
        sigma = 0.0;
        n = 0;
        for (i = 0; i < nobs; i++)
        {
          if (pep[i].protnum == protnum)
          {
             xmean += pep[i].vars[2*j] - pep[i].vars[2*j+1];
             sigma += (pep[i].vars[2*j] - pep[i].vars[2*j+1])*(pep[i].vars[2*j] - pep[i].vars[2*j+1]);
             n++;
          }
        }
        /* this still needs to be 2, p value will be 1 if not */
        if (n > 2)
        {
          xmean /= (float)n;
          sigma /= (float)(n-1);
          sigma += -(float)n*xmean*xmean/(float)(n-1);
          tval = xmean/(sqrt(sigma/(float)n));
          if (xmean != 0.0) pval = pvalue(tval, (float)(n-1));
          else pval = 1.0;
        }
        else pval = 1.0;
        if (j == 0) prot[protnum].pval1 = pval;
        if (j == 1) prot[protnum].pval2 = pval;
        if (j == 2) prot[protnum].pval3 = pval;
        if (j == 3) prot[protnum].pval4 = pval;
        protnum++;
      }
    }    
    
    /* write out peptide data */
    for (i = 0; i < nobs; i++)
    {
      if (((TYPE == 4) && (pep[i].speciesnum != 2) && (prot[pep[i].protnum].protnum >= NPEP)) || ((pep[i].prottype == TYPE) && (prot[pep[i].protnum].protnum >= NPEP)))
      {
        fprintf(fout1, "%s %d ", pep[i].name, pep[i].protnum);
        for (k = 0; k < nvar; k++)
        {
          fprintf(fout1, "%f ", pep[i].vars[k]);
        }      
        fprintf(fout1,"\n");
      }
    }
    fclose(fout1);
    
    /* write out protein (mean peptide) data */
    int test;
    for (i = 0; i < nprot; i++)
    {
      if (((TYPE == 4) && (prot[i].speciesnum != 2) && (prot[i].protnum >= NPEP)) || ((prot[i].prottype == TYPE) && (prot[i].protnum >= NPEP)))
      {
         fprintf(fout2, "%s ", prot[i].name);
        for (k = 0; k < nvar; k++)
        {
          fprintf(fout2, "%f ", prot[i].vars[k]);
        }      
        fprintf(fout2,"\n");
      }
    }
    fclose(fout2);
    
    /* write out t-test results */
    for (i = 0; i < nprot; i++)
    {
      if (((TYPE == 4) && (prot[i].speciesnum != 2) && (prot[i].protnum >= NPEP)) ||   ((prot[i].prottype == TYPE) && (prot[i].protnum >= NPEP)))
      {
        fprintf(fout3, "%s %d %d %d ", prot[i].name, i, prot[i].protnum, prot[i].speciesnum);
        fprintf(fout3, "%f %f ", prot[i].meanpval, prot[i].pval1to3);
        if (prot[i].pval1 != -1) 
        {
          fprintf(fout3, "%f %f %f %f ", prot[i].pval1, prot[i].pval2, prot[i].pval3, prot[i].pval4);
          if ((prot[i].pval1 < 0.05) && (prot[i].pval2 < 0.05) && (prot[i].pval3 < 0.05) && (prot[i].pval4 >= 0.05)) fprintf(fout3, "*\n");
          else if ((prot[i].pval1 < 0.05) && (prot[i].pval2 < 0.05) && (prot[i].pval3 < 0.05) && (prot[i].pval4 < 0.05)) fprintf(fout3, "**\n");
          else fprintf(fout3, "\n");
        }
        else if (prot[i].pval1to3 < 0.05) fprintf(fout3, "#\n");
      }
    }
    fclose(fout3);


    for (i = 0; i < nobs; i++)
    {
      free(pep[i].vars);
    }
    free(pep);
    for (i = 0; i < nprot; i++)
    {
      free(prot[i].vars);
    }
    free(prot);
    return 0;
}


/************************************************************************************************************************* 
code for calculating p-values below here 
**************************************************************************************************************************/
float gammln(float xx)
/* Returns the value ln[Γ(xx)] for xx > 0 */
{
  /* Internal arithmetic will be done in double precision, a nicety that you can omit if ﬁve-ﬁgure accuracy is good enough. */
  double x,y,tmp,ser;
  static double cof[6]={76.18009172947146,-86.50532032941677,24.01409824083091,-1.231739572450155,0.1208650973866179e-2,-0.5395239384953e-5};
  int j;
  y=x=xx;
  tmp=x+5.5;
  tmp -= (x+0.5)*log(tmp);
  ser=1.000000000190015;
  for (j=0;j<=5;j++) ser += cof[j]/++y;
  return -tmp+log(2.5066282746310005*ser/x);
}

float betacf(float a, float b, float x)
/* Used by betai: Evaluates continued fraction for incomplete beta function by modiﬁed Lentz’s method (§5.2) */
{
  int m,m2;
  float aa,c,d,del,h,qab,qam,qap;
  qab=a+b; /* These q’s will be used in factors that occur */
  qap=a+1.0; /* in the coeﬃcients (6.4.6) */
  qam=a-1.0;
  c=1.0; /* First step of Lentz’s method */
  d=1.0-qab*x/qap;
  if (fabs(d) < FPMIN) 
    d=FPMIN;
  d=1.0/d;
  h=d;
  for (m=1;m<=MAXIT;m++) {
    m2=2*m;
    aa=m*(b-m)*x/((qam+m2)*(a+m2));
    d=1.0+aa*d; //* One step (the even one) of the recurrence */
    if (fabs(d) < FPMIN) d=FPMIN;
    c=1.0+aa/c;
    if (fabs(c) < FPMIN) c=FPMIN;
    d=1.0/d;
    h *= d*c;
    aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
    d=1.0+aa*d; /* Next step of the recurrence (the odd one) */
    if (fabs(d) < FPMIN) d=FPMIN;
    c=1.0+aa/c;
    if (fabs(c) < FPMIN) c=FPMIN;
    d=1.0/d;
    del=d*c;
    h *= del;
    if (fabs(del-1.0) < EPS) break; /* Are we done? */
  }
  if (m > MAXIT) printf("a or b too big, or MAXIT too small in betacf\n");
  return h;
}

float betai(float a, float b, float x)
/* Returns the incomplete beta function Ix(a, b) */
{
  float betacf(float a, float b, float x);
  float gammln(float xx);
  float bt;
  if (x < 0.0 || x > 1.0) 
    printf("Bad x in routine betai\n");
  if (x == 0.0 || x == 1.0) 
    bt=0.0;
  else /* Factors in front of the continued fraction. */
    bt=exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.0-x));
  if (x < (a+1.0)/(a+b+2.0)) /* Use continued fraction directly. */
    return bt*betacf(a,b,x)/a;
  else /* Use continued fraction after making the symmetry transformation. */
    return 1.0-bt*betacf(b,a,1.0-x)/b; 
}

float pvalue( float t, float df ) 
/* Compute p-value of t-statistic */
{
  return betai(0.5*df,0.5,df/(df+t*t));  
}



       
